using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class BackgroundScroller : MonoBehaviour
{
    [SerializeField] Camera m_myCamera;
    [SerializeField] GameObject m_myObject;
    [SerializeField] int m_copies = 3;
    [SerializeField] float m_scrollSpeed = -1;
    [SerializeField] bool m_randomMirror = false;
    Vector2 m_CameraBoundsExtent;
    Vector2 m_ObjectBoundsExtent;
    List<GameObject> m_scrollObjects;


    // Start is called before the first frame update
    void Start()
    {
        m_CameraBoundsExtent = new Vector2(m_myCamera.orthographicSize * Screen.width / Screen.height, m_myCamera.orthographicSize);

        var br = m_myObject.GetComponent<BoundsRetreiver>();
        m_ObjectBoundsExtent = br.GetBoundsExtent();
        m_myObject.transform.position = new Vector3();

        m_scrollObjects = new List<GameObject>(m_copies);
        m_scrollObjects.Add(m_myObject);

        for (int i = 1; i < m_copies; i++)
        {
            m_scrollObjects.Add(Instantiate(m_myObject, transform));
            m_scrollObjects[i].transform.position += new Vector3(m_ObjectBoundsExtent.x * 2 * i, 0);
        }
    }

    // Update is called once per frame
    void Update()
    {
        for (int i = 0; i < m_copies; i++)
        {
            m_scrollObjects[i].transform.position += new Vector3(m_scrollSpeed * Time.deltaTime, 0);

            //only checks if left bounds in negative x region
            if (m_scrollObjects[i].transform.position.x <
                (m_myCamera.transform.position.x
                - m_CameraBoundsExtent.x
                - m_ObjectBoundsExtent.x))
            {
                if (m_randomMirror)
                {
                    int r = Random.Range(0, 3);
                    switch (r)
                    {
                        case 1:
                            m_scrollObjects[i].transform.localScale =
                                new Vector3(m_scrollObjects[i].transform.localScale.x * -1,
                                m_scrollObjects[i].transform.localScale.y,
                                m_scrollObjects[i].transform.localScale.z);
                            break;
                        case 2:
                            m_scrollObjects[i].transform.localScale =
                                new Vector3(m_scrollObjects[i].transform.localScale.x,
                                m_scrollObjects[i].transform.localScale.y * -1,
                                m_scrollObjects[i].transform.localScale.z);
                            break;
                    }
                }
                m_scrollObjects[i].transform.position += new Vector3(m_ObjectBoundsExtent.x*2*m_copies, 0, 0);
            }
        }
    }
}
