using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using MEC;

public class Enemy_Move : MonoBehaviour
{
    [SerializeField] float m_shipSpeed = 8;
    [SerializeField] float m_acceleration = 8;
    Rigidbody2D rbody;

    private void Awake()
    {
        rbody = GetComponent<Rigidbody2D>();
    }

    public void StartMoveTo(Vector3 location, System.Action callback)
    {
        Timing.RunCoroutine(MoveTo(location, callback).CancelWith(gameObject));
    }

    public IEnumerator<float> MoveTo(Vector3 location, System.Action callback)
    {
        Vector3 direction = (location - transform.position);
        Vector2 dir2d = new Vector2(direction.x, direction.y).normalized;
        while(rbody.velocity.sqrMagnitude <= m_shipSpeed * m_shipSpeed)
        {
            rbody.velocity += m_acceleration * Time.deltaTime * dir2d;
            yield return Timing.WaitForOneFrame;
        }
        rbody.velocity = m_shipSpeed * dir2d;
        while ((location - transform.position).magnitude > (rbody.velocity.sqrMagnitude/(2*m_acceleration)))
        {
            yield return Timing.WaitForOneFrame;
        }

        while (rbody.velocity.sqrMagnitude > 0)
        {
            Vector2 speedDecrease = (m_acceleration * Time.deltaTime * dir2d);
            if (rbody.velocity.sqrMagnitude > speedDecrease.sqrMagnitude)
            {
                rbody.velocity -= speedDecrease;
            }
            else
            {
                rbody.velocity = new Vector2();
            }
            yield return Timing.WaitForOneFrame;
        }   
        
        rbody.velocity = new Vector2(0, 0);
        transform.position = location;

        callback();
    }

    CoroutineHandle m_followCoroutine;
    public void StartFollow(Transform player, float idealRange)
    {
        m_followCoroutine = Timing.RunCoroutine(FollowPlayer(player, idealRange).CancelWith(gameObject));
    }

    public void EndFollow()
    {
        Timing.KillCoroutines(m_followCoroutine);
        rbody.velocity = Vector2.zero;
    }

    IEnumerator<float> FollowPlayer(Transform player, float idealRange)
    {
        while(player != null)
        {
            Vector3 direction = (player.position - transform.position);
            if (Mathf.Abs(transform.position.y - player.position.y) > idealRange)
            {
                Vector2 dir2d = new Vector2(0, direction.y).normalized;
                rbody.velocity = dir2d * m_shipSpeed;
            }
            else
            {
                rbody.velocity = new Vector2();
            }
            yield return Timing.WaitForOneFrame;
        }
    }
}
