using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using TMPro;
using UnityEngine.SceneManagement;

public class MainSceneManager : MonoBehaviour
{
    [SerializeField] Wave_Spawner m_wave_Spawner;
    [SerializeField] PromptsSystem m_promptsSystem;
    [SerializeField] PS_Health m_playerHealth;
    [SerializeField] float m_playerTime = 60;
    [SerializeField] TextMeshProUGUI m_timerText;
    [SerializeField] GameObject GameOverScreen;
    [SerializeField] GameObject GameWinScreen;

    [SerializeField] List<BulletPoolData> bulletPools;

    uint m_gameTimeID;
    bool gameEnd = false;
    TimerData m_td;

    private void Start()
    {
        bulletPools.ForEach((x) => { x.Reset(); });

        m_playerHealth.OnPlayerHit += M_playerHealth_OnPlayerHit;
        m_gameTimeID = Timer_Manager.AddTimer(gameObject, TimerRunOut, m_playerTime);
        m_td = Timer_Manager.GetTimerData(m_gameTimeID);

        m_wave_Spawner.StartSequence();
        m_promptsSystem.StartPrompts();
    }

    private void Update()
    {
        if(!gameEnd)
        {
            if(m_promptsSystem.Points >= m_promptsSystem.MaxPoints)
            {
                GameSucceed();
            }

            if(m_td != null)
            {
                m_timerText.text = string.Format("{0:00.00}s", Mathf.Clamp(m_playerTime - m_td.TimeElapsed(),0, m_playerTime));
            }
        }
    }

    private void M_playerHealth_OnPlayerHit(int health)
    {
        if(health == 0)
        {
            GameOver();
        }
    }

    void TimerRunOut()
    {
        GameOver();
    }

    void GameOver()
    {
        PauseGame();
        GameOverScreen.SetActive(true);
        gameEnd = true;
    }

    void GameSucceed()
    {
        PauseGame();
        GameWinScreen.SetActive(true);
        gameEnd = true;
    }

    void PauseGame()
    {
        Time.timeScale = 0;
    }

    void UnpauseGame()
    {
        Time.timeScale = 1;
    }    
}
