using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using MEC;

public class PS_Health : MonoBehaviour
{
    public event System.Action<int> OnPlayerHit;
    public event System.Action<int> OnHealthChange;
    [SerializeField] int m_shipHealth = 3;
    [SerializeField] float m_invincibilityTime = 3;
    [SerializeField] float m_invincibilityFlashSpeed = 12;
    [SerializeField] AudioSource source;
    [SerializeField] AudioClip hurt;
    bool m_invincibility = false;

    uint m_timerID;

    public int ShipHealth
    {
        get { return m_shipHealth; }
    }

    public void DamagePlayer()
    {
        if (!m_invincibility)
        {
            source.PlayOneShot(hurt);
            m_shipHealth--;
            Debug.Log($"Player hit: {m_shipHealth}");
            OnPlayerHit?.Invoke(m_shipHealth);
            OnHealthChange?.Invoke(m_shipHealth);

            m_invincibility = true;
            Timer_Manager.AddTimer(gameObject, () => { m_invincibility = false; }, m_invincibilityTime);
            Timing.RunCoroutine(RunInvincibility().CancelWith(gameObject));
        }
    }

    public void AddHealth()
    {
        m_shipHealth++;
        OnHealthChange?.Invoke(m_shipHealth);
    }

    IEnumerator<float> RunInvincibility()
    {
        var sprites = new List<SpriteRenderer>(GetComponentsInChildren<SpriteRenderer>());
        while (m_invincibility)
        {
            sprites.ForEach((x) =>
            {
                x.color = x.color.a >= 1 ? 
                new Color(x.color.r, x.color.b, x.color.g, 0) 
                : new Color(x.color.r, x.color.b, x.color.g, 1);
            });
            yield return Timing.WaitForSeconds(1 / m_invincibilityFlashSpeed);
        }
    }

    private void OnDestroy()
    {
        Timer_Manager.CancelTimer(m_timerID);
    }
}
