using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using MEC;

public class PS_Move : MonoBehaviour
{
    [SerializeField] float m_speed = 1;

    [Header("Dash Settings")]
    [SerializeField] AnimationCurve m_dashSpeed;
    [SerializeField] float m_dashTime = 0.5f;
    [SerializeField] float m_dashWindow = 0.2f;
    bool m_isDashing = false;
    bool m_enteredDashWindow = false;
    Vector2 m_prevMoveDir;
    Vector2 m_dashDirection;
    uint m_dashWindowTimerId;

    private Rigidbody2D m_rBody;

    // Start is called before the first frame update
    void Start()
    {
        m_rBody = GetComponent<Rigidbody2D>();
    }

    // Update is called once per frame
    void Update()
    {
        if (!m_isDashing)
        {
            float vertical = Input.GetAxis("Vertical");
            float horizontal = Input.GetAxis("Horizontal");
            Vector2 direction = new Vector2(horizontal, vertical).normalized;
            m_rBody.velocity = direction * m_speed;

            if(Input.GetButtonDown("Vertical") || Input.GetButtonDown("Horizontal"))
            {
                if (m_enteredDashWindow)
                {
                    if (m_prevMoveDir == direction)
                    {
                        m_enteredDashWindow = false;
                        Timer_Manager.CancelTimer(m_dashWindowTimerId);
                        m_dashDirection = direction;
                        Timing.RunCoroutine(Dash());
                    }
                }
                else
                {
                    m_enteredDashWindow = true;
                    m_dashWindowTimerId = Timer_Manager.AddTimer(gameObject, () => { m_enteredDashWindow = false; }, m_dashWindow);
                }
                m_prevMoveDir = direction;
            }
        }
    }

    IEnumerator<float> Dash()
    {
        float time = m_dashTime;
        m_isDashing = true;
        while(time>=0)
        {
            m_rBody.velocity = m_dashSpeed.Evaluate(1 - (time / m_dashTime)) * m_dashDirection;
            time -= Time.deltaTime;
            yield return Timing.WaitForOneFrame;
        }
        m_isDashing = false;
    }
}
