using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class PrefabPool
{
    private GameObject prefab;
    private List<GameObject> m_active, m_inactive;

    public PrefabPool(GameObject prefab, int pool_count)
    {
        m_active = new List<GameObject>(pool_count);
        m_inactive = new List<GameObject>(pool_count);

        for (int i = 0; i < pool_count; i++)
        {
            GameObject go = GameObject.Instantiate(prefab); 
            m_inactive.Add(go);
            go.SetActive(false);
            go.AddComponent<PoolInactiveNotify>().OnInactive += ObjectReturn;
        }
    }

    public void ObjectReturn(GameObject Go)
    {
        m_active.Remove(Go);
        m_inactive.Add(Go);
    }

    public GameObject Next()
    {
        GameObject go = null;
        if(m_inactive.Count>0)
        {
            go = m_inactive[m_inactive.Count - 1];
            m_active.Add(m_inactive[m_inactive.Count-1]);
            m_inactive.RemoveAt(m_inactive.Count - 1);
        }
        else
        {
            go = GameObject.Instantiate(prefab);
            m_active.Add(go);
            go.AddComponent<PoolInactiveNotify>();
        }

        return go;
    }

    public void Clear()
    {
        m_active.Clear();
        m_inactive.Clear();
    }
}
