using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class PromptsSystem : MonoBehaviour
{
    List<Prompts> m_myPrompts;
    Prompts m_curPrompt;
    uint m_timerID;
    [SerializeField]float m_timeBetweenPrompts = 0.5f;
    [SerializeField]bool m_startAutomatically = false;

    int m_points = 0;
    [SerializeField] int m_maxPoints = 50;

    [SerializeField] UnityEngine.UI.Image m_pointsBar;

    public int Points
    {
        get { return m_points; }
    }

    public int MaxPoints
    {
        get { return m_maxPoints; }
    }

    private void Start()
    {
        m_pointsBar.fillAmount = (float)m_points / (float)m_maxPoints;
        if (m_startAutomatically)
        {
            StartPrompts();
        }
    }


    public void StartPrompts()
    {
        m_myPrompts = new List<Prompts>(GetComponentsInChildren<Prompts>());
        NextPrompt();
    }

    public void EndPrompts()
    {
        m_curPrompt?.CancelPrompt();
        Timer_Manager.CancelTimer(m_timerID);
    }

    void NextPrompt()
    {
        m_timerID =  Timer_Manager.AddTimer(gameObject, () =>
        {
            m_curPrompt = m_myPrompts[Random.Range(0, m_myPrompts.Count)];
            m_curPrompt.StartPrompt(CompletePrompt);
        }, m_timeBetweenPrompts);
    }

    void CompletePrompt(bool succeed, int pointsGained)
    {
        if(succeed)
        {
            m_points += pointsGained;
            m_points = Mathf.Min(m_points, m_maxPoints);
            m_pointsBar.fillAmount = (float)m_points / (float)m_maxPoints;
        }

        NextPrompt();
    }
}
