using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using System.Linq;

public class ShipGun : MonoBehaviour
{
    [SerializeField]Transform m_bulletFireParent;
    [SerializeField] float m_baseFireRate = 2f;
    [SerializeField] BulletPoolData m_bulletPoolData;
    private List<Transform> m_bulletFireTransforms;
    private bool m_canFire = true;

    uint m_timerId;

    private void OnEnable()
    {
        FireTransformsGather();
    }

    public void FireBullet()
    {
        if (m_canFire)
        {
            m_canFire = false;
            m_timerId = Timer_Manager.AddTimer(gameObject, FireCooldownComplete, 1 / m_baseFireRate);

            m_bulletFireTransforms.ForEach((x) =>
            {
                GameObject nextBullet = m_bulletPoolData.PrefabPool.Next();
                nextBullet.SetActive(true);
                nextBullet.transform.position = x.position;
                nextBullet.transform.rotation = x.rotation;
            });
        }
    }

    void FireCooldownComplete()
    {
        m_canFire = true;
    }

    void FireTransformsGather()
    {
        if(m_bulletFireParent != null)
        {
            m_bulletFireTransforms = new List<Transform>(m_bulletFireParent.GetComponentsInChildren<Transform>()).Where(x=> x.gameObject.activeInHierarchy && x != m_bulletFireParent).ToList();
        }
    }

    private void OnDestroy()
    {
        Timer_Manager.CancelTimer(m_timerId);
    }
}
