using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class Timer_Manager : MonoBehaviour
{
    private static Timer_Manager m_instance;
    private List<TimerData> m_timerHandles = new List<TimerData>();
    private uint incrementID = 0;

    public static Timer_Manager Instance
    {
        get
        {
            if(m_instance == null)
            {
                m_instance = GameObject.FindObjectOfType<Timer_Manager>();
                if (m_instance != null)
                {
                    m_instance.m_timerHandles = new List<TimerData>();
                }
            }
            return m_instance;
        }
    }

    public static uint AddTimer(GameObject caller, System.Action callableAction, float time)
    {
        TimerData th = new TimerData
        {
            callingObject = caller,
            id = Instance.incrementID++,
            action = callableAction,
            timeStarted = Time.time,
            timeAmount = time
        };
        Instance.m_timerHandles.Add(th);

        return th.id;
    }

    public static TimerData GetTimerData(uint id)
    {
        if (Instance != null)
        {
            return Instance.m_timerHandles.Find((x) => x.id == id);
        }
        return null;
    }

    public static void CancelTimer(uint ID)
    {
        if (Instance != null)
        {
            Instance.m_timerHandles.RemoveAll((x) => x.id == ID);
        }
    }

    // Update is called once per frame
    void Update()
    {
        List<TimerData> td = new List<TimerData>(m_timerHandles);
        foreach (var timerHandle in td)
        {
             if(timerHandle.TimeElapsed() >= timerHandle.timeAmount)
            {
                if (timerHandle.callingObject != null)
                {
                    timerHandle.action?.Invoke();
                }
            }
        }

        m_timerHandles.RemoveAll((x) => x.TimeElapsed() >= x.timeAmount || x.callingObject == null);
    }

    void OnDestroy()
    {
        m_timerHandles.Clear();
    }
}

public class TimerData
{
    public uint id;
    public float timeAmount;
    public float timeStarted;
    public System.Action action;
    public GameObject callingObject;

    public float TimeElapsed()
    {
        return Time.time - timeStarted;
    }
}
